! file src_equations/equations_module_template.f90
!
! Copyright 2009-2011 Dalton Harvie (daltonh@unimelb.edu.au)
! 
! This file is part of arb finite volume solver, referred to as `arb'.
! 
! arb is a software package designed to solve arbitrary partial
! differential equations on unstructured meshes using the finite volume
! method.  Primarily it consists of fortran source code, perl source
! code and shell scripts.  arb replies on certain third party software
! to run, most notably the computer algebra system maxima
! <http://maxima.sourceforge.net/> which is released under the GNU GPL.
! 
! The copyright of arb is held by Dalton Harvie.
! 
! arb is released under the GNU GPL.  arb is free software: you can
! redistribute it and/or modify it under the terms of the GNU General
! Public License (version 3) as published by the Free Software Foundation.
! You should have received a copy of the GNU General Public License
! along with arb (see file license/gpl.txt after unpacking).  If not,
! see <http://www.gnu.org/licenses/>.
! 
! arb is distributed in the hope that it will be useful, but WITHOUT
! ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
! FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
! for more details.
! 
! For full details of arb's license see the license directory.
! 
! The current homepage for the arb finite volume solver project is
! <http://www.chemeng.unimelb.edu.au/people/staff/daltonh/downloads/arb>.
!
!-------------------------------------------------------------------------
!
! this file has strings replaced by the script setup_equations.pl to form
! the fortran code equations_module.f90
!
module equations_module

contains

!----------------------------------------------------------------------------

subroutine allocate_meta_arrays

! here we allocate array elements of the meta data that corresponds to the input files

use general_module
logical, parameter :: debug = .false.
                  
if (debug) write(*,'(80(1h+)/a)') 'subroutine allocate_meta_arrays'

! allocate meta arrays
<sub_string:allocate_meta_arrays>

! setup someloop which is also used locally within update subroutines
<sub_string:allocate_someloop>

if (debug) write(*,'(a/80(1h-))') 'subroutine allocate_meta_arrays'

end subroutine allocate_meta_arrays

!----------------------------------------------------------------------------

subroutine update

! here we update all the deriveds and equations

use general_module
integer :: nvar, m, ns, i, j
double precision :: deriv
character(len=1000) :: formatline
logical, parameter :: debug = .false.
                  
if (debug) write(*,'(80(1h+)/a)') 'subroutine update'

if (debug) write(*,'(40(1h+)/a)') 'derived updates'

do nvar = 1, allocatable_size(var_list(var_list_number(centring="all",type="derived"))%list)
  m = var_list(var_list_number(centring="all",type="derived"))%list(nvar)

  i = 0
  j = 0

  if (debug) write(*,*) 'updating var derived: m = ',m,': name = ',trim(var(m)%name),': centring = ',var(m)%centring

! equations
  <sub_string:derived>

  if (debug) then
    formatline = '(a,'//trim(indexformat)//',a,i3,a,a)'
    do ns = 1, ubound(var(m)%funk,1)
      write(*,fmt=formatline) '(ns,m) = (',ns,',',m,'): var = ',trim(print_funk(var(m)%funk(ns)))
    end do
  end if

end do

if (debug) write(*,'(a/40(1h-))') 'derived updates'

!------------------------------------------------------------------

if (debug) write(*,'(40(1h+)/a)') 'equation updates'

do nvar = 1, allocatable_size(var_list(var_list_number(centring="all",type="equation"))%list)
  m = var_list(var_list_number(centring="all",type="equation"))%list(nvar)

  i = 0
  j = 0

  if (debug) write(*,*) 'updating var equation: m = ',m,': name = ',trim(var(m)%name),': centring = ',var(m)%centring

! equations
  <sub_string:equation>

  if (debug) then
    formatline = '(a,'//trim(indexformat)//',a,i3,a,a)'
    do ns = 1, ubound(var(m)%funk,1)
      write(*,fmt=formatline) '(ns,m) = (',ns,',',m,'): var = ',trim(print_funk(var(m)%funk(ns)))
    end do
  end if

end do

if (debug) write(*,'(a/40(1h-))') 'equation updates'

if (debug) write(*,'(a/80(1h-))') 'subroutine update'

end subroutine update

!----------------------------------------------------------------------------

recursive subroutine update_someloop(m,ilast,jlast)

! here we update each of all the possible someloops
! bug wife again!!!

use general_module
integer :: m, ilast, jlast, i, j, ns, region_number, flux_direction, to_ns, from_ns
double precision :: deriv
logical, parameter :: debug = .false.
                  
if (debug) write(*,'(80(1h+)/a)') 'subroutine update_someloop'

i = ilast
j = jlast

call reset_funk(someloop(m))

<sub_string:someloop>

if (debug) write(*,'(a/80(1h-))') 'subroutine update_someloop'

end subroutine update_someloop

!----------------------------------------------------------------------------

subroutine update_constants

! here we update all the constants that are calculated using equations

use general_module
use gmsh_module
integer :: nvar, m, ns, i, j
double precision :: deriv
character(len=1000) :: formatline
logical, parameter :: debug = .false.
                  
if (debug) write(*,'(80(1h+)/a)') 'subroutine update_constants'

do nvar = 1, allocatable_size(var_list(var_list_number(centring="all",type="constant"))%list)
  m = var_list(var_list_number(centring="all",type="constant"))%list(nvar)

  i = 0
  j = 0

  if (debug) write(*,*) 'updating var constant: m = ',m,': name = ',trim(var(m)%name),': centring = ',var(m)%centring

! equations
  <sub_string:constant>

  call read_gmesh(contents='data',var_number=m)

  if (debug) then
    formatline = '(a,'//trim(indexformat)//',a,i3,a,a)'
    do ns = 1, ubound(var(m)%funk,1)
      write(*,fmt=formatline) '(ns,m) = (',ns,',',m,'): var = ',trim(print_funk(var(m)%funk(ns)))
    end do
  end if

end do

if (debug) write(*,'(a/80(1h-))') 'subroutine update_constants'

end subroutine update_constants

!----------------------------------------------------------------------------

subroutine update_unknowns

! here we update all the unknown initial values

use general_module
use gmsh_module
integer :: nvar, m, ns, i, j
double precision :: deriv
character(len=1000) :: formatline
logical, parameter :: debug = .false.
                  
if (debug) write(*,'(80(1h+)/a)') 'subroutine update_unknowns'

do nvar = 1, allocatable_size(var_list(var_list_number(centring="all",type="unknown"))%list)
  m = var_list(var_list_number(centring="all",type="unknown"))%list(nvar)

  i = 0
  j = 0

  if (debug) write(*,*) 'updating var unknown: m = ',m,': name = ',trim(var(m)%name),': centring = ',var(m)%centring

! equations
  <sub_string:unknown>

  call read_gmesh(contents='data',var_number=m)

  if (debug) then
    formatline = '(a,'//trim(indexformat)//',a,i3,a,a)'
    do ns = 1, ubound(var(m)%funk,1)
      write(*,fmt=formatline) '(ns,m) = (',ns,',',m,'): var = ',trim(print_funk(var(m)%funk(ns)))
    end do
  end if

end do

if (debug) write(*,'(a/80(1h-))') 'subroutine update_unknowns'

end subroutine update_unknowns

!----------------------------------------------------------------------------

subroutine update_outputs

! here we update all the outputs that are calculated using equations

use general_module
integer :: nvar, m, ns, i, j
double precision :: deriv
character(len=1000) :: formatline
logical, parameter :: debug = .false.
                  
if (debug) write(*,'(80(1h+)/a)') 'subroutine update_outputs'

do nvar = 1, allocatable_size(var_list(var_list_number(centring="all",type="output"))%list)
  m = var_list(var_list_number(centring="all",type="output"))%list(nvar)

  i = 0
  j = 0

  if (debug) write(*,*) 'updating var output: m = ',m,': name = ',trim(var(m)%name),': centring = ',var(m)%centring

! equations
  <sub_string:output>

  if (debug) then
    formatline = '(a,'//trim(indexformat)//',a,i3,a,a)'
    do ns = 1, ubound(var(m)%funk,1)
      write(*,fmt=formatline) '(ns,m) = (',ns,',',m,'): var = ',trim(print_funk(var(m)%funk(ns)))
    end do
  end if

end do

if (debug) write(*,'(a/80(1h-))') 'subroutine update_outputs'

end subroutine update_outputs

!----------------------------------------------------------------------------

subroutine variable_checker(error)

! here we check on the magnitudes of any variables against specified options

use general_module
integer :: ns, error
logical, parameter :: debug = .false.
                  
if (debug) write(*,'(80(1h+)/a)') 'subroutine variable_checker'

<sub_string:variable_checker>

if (debug) write(*,'(a/80(1h-))') 'subroutine variable_checker'

end subroutine variable_checker

!----------------------------------------------------------------------------

end module equations_module

!----------------------------------------------------------------------------
