! file src/main.f90
!
! Copyright 2009-2011 Dalton Harvie (daltonh@unimelb.edu.au)
! 
! This file is part of arb finite volume solver, referred to as `arb'.
! 
! arb is a software package designed to solve arbitrary partial
! differential equations on unstructured meshes using the finite volume
! method.  Primarily it consists of fortran source code, perl source
! code and shell scripts.  arb replies on certain third party software
! to run, most notably the computer algebra system maxima
! <http://maxima.sourceforge.net/> which is released under the GNU GPL.
! 
! The copyright of arb is held by Dalton Harvie.
! 
! arb is released under the GNU GPL.  arb is free software: you can
! redistribute it and/or modify it under the terms of the GNU General
! Public License (version 3) as published by the Free Software Foundation.
! You should have received a copy of the GNU General Public License
! along with arb (see file license/gpl.txt after unpacking).  If not,
! see <http://www.gnu.org/licenses/>.
! 
! arb is distributed in the hope that it will be useful, but WITHOUT
! ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
! FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
! for more details.
! 
! For full details of arb's license see the license directory.
! 
! The current homepage for the arb finite volume solver project is
! <http://www.chemeng.unimelb.edu.au/people/staff/daltonh/downloads/arb>.
!
!-------------------------------------------------------------------------
program arb
                  
! written in f90 with object oriented approach, hopefully
! daltonh, 070608

use general_module
use setup_module
use equations_module
use solver_module
use output_module

implicit none
character(len=100) :: formatline
double precision :: newtres=1.d+10
integer :: error, nstep = 0
logical :: stopfile = .false.
logical, parameter :: debug = .true.

!---------------------------------------------------

formatline = "(a,f4.2,a,a,a)"
write(*,fmt=formatline) 'program arb, version ',version,' (',trim(versionname),'), written by dalton harvie'

call time_process
call setup ! sets up variable metadata, reads in values, allocates arrays, creates mesh, initialises fields etc
call time_process(description='setup')

call update
call update_checker(error,newtres)
call time_process(description='initial update')

call output(nstep)

if (error /= 0) stop "ERROR: problem after first update call"

formatline = "(a,g8.3,a,g8.3)"
write(*,fmt=formatline) 'INFO: after setup: newtres = ',newtres,': newtres/newtrestol = ', &
  newtres/newtrestol

newton_solver_loop: do while (newtres > newtrestol .and. nstep < newtstepmax )

  nstep = nstep + 1

  call newtsolver(error,newtres) ! uses newton's method to solve equations

  if (error /= 0) exit newton_solver_loop

  formatline = "(a,i6,a,g8.3,a,g8.3)"
  write(*,fmt=formatline) 'INFO: nstep = ',nstep,': newtres = ',newtres,': newtres/newtrestol = ', &
    newtres/newtrestol

  inquire(file='stop',exist=stopfile)
  if (stopfile) then
    write(*,'(a)') 'INFO: user initiated simulation stop'
    call unlink('./stop')
    exit
  end if
  
end do newton_solver_loop

call time_process
call output(nstep)
call time_process(description='output')

if (error /= 0) then
  write(*,'(a)') 'ERROR: problem in newtsolver'
else if (newtres > newtrestol) then
  write(*,'(a)') 'ERROR: solver did not converge'
else
  write(*,'(a)') 'INFO: run successfully completed'
end if

end program arb

!-----------------------------------------------------------------
